#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2010 Łukasz Langa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for helpers."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

def setup():
    import os
    os.environ['DJANGO_SETTINGS_MODULE'] = 'langacore.kit.dummy'

def test_dummy():
    """Just see if the import works as expected."""
    from langacore.kit.django import helpers

def test_cut():
    from langacore.kit.django.helpers import cut
    assert "123456789" == cut("123456789", length=9)
    assert "123456789 (...)" == cut("123456789ABC", length=9)
    assert "123456789 ..." == cut("123456789ABC", length=9, trailing=" ...")
    assert "123456789ABC" == cut("123456789ABC", length=12, trailing=" ...")
    assert None == cut(None, length=12, trailing=" ...")
    assert "" == cut("", length=12, trailing=" ...")
    assert "" == cut("", length=0, trailing=" ...")
    assert "12345678 (...)" == cut("123456789", length=-1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2010 Łukasz Langa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""langacore.kit.django.common.models
   ----------------------------------

   Contains a small set of useful abstract model base classes that are not
   application-specific.
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from datetime import datetime

from django.conf import settings
from django.core.mail import send_mail
from django.db import models as db
from django.utils.translation import ugettext_lazy as _

from langacore.kit.django.choices import Language


class Named(db.Model):
    """Describes an abstract model with a ``name`` field."""

    name = db.CharField(verbose_name=_("name"), max_length=50, unique=True)

    class Meta:
        abstract = True

    def __unicode__(self):
        return self.name


class Titled(db.Model):
    """Describes an abstract model with a ``title`` field."""

    title = db.CharField(verbose_name=_("title"), max_length=100, unique=True)

    class Meta:
        abstract = True

    def __unicode__(self):
        return self.title


class Slugged(db.Model):
    """Describes an abstract model with a ``slug`` field."""

    slug = db.SlugField(verbose_name=_("permalink"), unique=True)

    class Meta:
        abstract = True

    def __unicode__(self):
        return self.slug


class TimeTrackable(db.Model):
    """Describes an abstract model whose lifecycle is tracked by time. Includes
    a ``created`` field that is set automatically upon object creation and
    a ``modified`` field that is set automatically upon calling ``save()`` on
    the object.
    """
    created = db.DateTimeField(verbose_name=_("date created"),
        default=datetime.now)
    modified = db.DateTimeField(verbose_name=_("last modified"),
        default=datetime.now)

    class Meta:
        abstract = True

    def save(self):
        self.modified = datetime.now()
        super(TimeTrackable, self).save()


class Localized(db.Model):
    """Describes an abstract model which holds data in a specified
    ``language``. The language is chosen from the Language choices class
    but only from those specified in settings.LANGUAGES. The default value
    is settings.LANGUAGE_CODE."""
    language = db.PositiveIntegerField(verbose_name=_("language"),
        choices=Language(filter=set([lang[0] for lang in settings.LANGUAGES])),
            default=Language.IDFromName(settings.LANGUAGE_CODE))

    class Meta:
        abstract = True


class MonitoredActivity(db.Model):
    """Describes an abstract model which holds the timestamp of last user
    activity on the site. Activity is logged using the ActivityMiddleware."""
    last_active = db.DateTimeField(verbose_name=_("last active"),
        blank=True, null=True, default=None)

    _is_online_secs = getattr(settings, 'CURRENTLY_ONLINE_INTERVAL', 120)
    _was_online_secs = getattr(settings, 'RECENTLY_ONLINE_INTERVAL', 300)

    def is_currently_online(self, time_limit=_is_online_secs):
        """True if the user's last activity was within the last `time_limit`
        seconds (default value 2 minutes, customizable by the
        ``CURRENTLY_ONLINE_INTERVAL`` setting."""
        return (bool(self.last_active) and
            (datetime.now() - self.last_active).seconds <= time_limit)

    def was_recently_online(self, time_limit=_was_online_secs):
        """True if the user's last activity was within the last `time_limit`
        seconds (default value 5 minutes, customizable by the
        ``RECENTLY_ONLINE_INTERVAL`` setting."""
        return self.is_currently_online(time_limit=time_limit)

    class Meta:
        abstract = True


class DisplayCounter(db.Model):
    """Describes an abstract model which display count can be incremented by
    calling ``bump()``. Models inheriting from ``DisplayCounter`` can define
    a special ``bump_save()`` method which is called instead of the default
    ``save()`` on each ``bump()`` (for instance to circumvent updating the
    ``modified`` field if the model is also ``TimeTrackable``.
    """
    display_count = db.PositiveIntegerField(verbose_name=_("display count"),
        default=0, editable=False)

    def bump(self):
        self.display_count += 1
        if not 'bump_save' in dir(self):
            self.save()
        else:
            self.bump_save()

    class Meta:
        abstract = True


class ViewableSoftDeletableManager(db.Manager):
    """An objet manager to automatically hide objects that were soft deleted
    for models inheriting ``SoftDeletable``."""

    def get_query_set(self):
        # get the original query set
        query_set = super(ViewableSoftDeletableManager, self).get_query_set()
        # leave rows which are deleted
        query_set = query_set.filter(deleted=False)
        return query_set


class SoftDeletable(db.Model):
    """
    Describes an abstract models which can be soft deleted, that is instead of
    actually removing objects from the database, they have a ``deleted`` field
    which is set to ``True`` and the object is then invisible in normal
    operations (thanks to ``ViewableSoftDeletableManager``).
    """
    deleted = db.BooleanField(verbose_name=_("deleted"), default=False)
    admin_objects = db.Manager()
    objects = ViewableSoftDeletableManager()

    class Meta:
        abstract = True


# For now this needs to be at the end of the file.
# FIXME: move it where it's supposed to be.

import os
from time import sleep
from threading import Thread

class MassMailer(Thread):
    """
    A thread that can be used to mail the specified profiles with a certain
    message. After every message it waits a specified time (by default
    a second).
    """
    def __init__ (self, profiles, subject, content, inverval=1.0, force=False):
        """Creates the thread.

        :param profiles: a sequence of profiles that are to be mailed

        :param subject: the subject of the message to be sent

        :param content: the actual content to be sent

        :param interval: number of seconds to wait after sending every message

        :param force: if ``True``, privacy settings of the users are
                      disregarded
        """

        Thread.__init__(self)
        self.profiles = profiles
        self.subject = subject
        self.content = content
        self.interval = interval
        self.force = force

    def run(self):
        print("Mailer subprocess started (%d)." % os.getpid())
        for profile in self.profiles:
            mail = profile.user.email
            # FIXME: check privacy
            send_mail(self.subject, self.content, None, [mail])
            print("Mailer subprocess (%d): sent mail to %s." % (os.getpid(), mail))
            sleep(self.interval)

# NO CODE BEYOND THIS POINT



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2010 Łukasz Langa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for langacore.kit.django
   ------------------------------

   ``py.test`` is required to run the tests."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2010 Łukasz Langa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""A set of string-related filters."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals


import re
from django.template import Library
from django.template.defaultfilters import slugify as crippled_slugify
from django.utils.safestring import mark_safe

from langacore.kit.i18n import translit

register = Library()


@register.filter
def title(text):
    """Like built-in \\|title filter but leaves existing caps alone, eg.
    ``'I work for the FBI'`` -> ``'I Work For The FBI'``.

    Template tag available in ``common`` app's ``strings`` library.
    """
    pattern=re.compile(r"(\s|^)\w")
    cap=lambda x: x.group().upper()

    return pattern.sub(cap, text + '') #+ '' to handle proxy objects


@register.filter
def transliterate(text, country_code, text_lang=''):
    """If the country with the specified code is using Cyrillic alphabet,
    transliterates Latin strings using ``langacore.kit.i18n.translit``.

    Template tag available in ``common`` app's ``strings`` library.
    """
    return translit.any(country_code, text, input_lang=text_lang)


@register.filter
def slugify(text, fallback='untitled-0'):
    """A version of the builtin ``django.template.defaultfilters`` which
    doesn't let empty strings as outcomes.

    Template tag available in ``common`` app's ``strings`` library.

    :param text: the text to slugify

    :param fallback: the text to return in case the builtin slugify returns an
                     empty value
    """

    slug = crippled_slugify(text)
    return slug if slug else fallback


@register.filter
def strike_empty(obj):
    """Converts any falsy value to ``u"---"``.

    Template tag available in ``common`` app's ``strings`` library.
    """

    return obj if obj else "---"


@register.filter
def nbsp(obj):
    """Converts spaces to nbsp entities.

    Template tag available in ``common`` app's ``strings`` library.
    """

    return mark_safe(re.sub(r"\s+", "&nbsp;", obj))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2010 Łukasz Langa

"""Various unrelated routines and helper classes."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import re

from django.conf import settings
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import render_to_response as _render_to_response
from django.template import RequestContext
from django.utils import simplejson
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext



DOTS_REGEX = re.compile(r'\.\s+')


def render(request, template_name, context, debug=False, mimetype=None):
    """Renders the `context` within for a `request` using a template named
    `template_name` returning the required `mimetype`."""

    if hasattr(settings, 'AUTH_PROFILE_MODULE'):
        if 'user_profile' in context:
            raise KeyError, ("langacore.kit.django.helpers.render() doesn't "
                             "accept contexts with 'user_profile'")
        if 'other_user_profile' in context:
            raise KeyError, ("langacore.kit.django.helpers.render() doesn't "
                             "accept contexts with 'other_user_profile'")

        context['user_profile'] = request.user.get_profile() \
                                  if request.user.is_authenticated() else None
        context['other_user_profile'] = context['other_user'].get_profile() \
                                        if 'other_user' in context else None

    if debug:
        for key in context:
            format = (key, str(context[key]), type(context[key]),
                      repr(context[key]))
            print("context['{}'] = {} (type: {}) (repr: {})".format(*format))
    return _render_to_response(template_name,
                               RequestContext(request, context),
                               mimetype=mimetype)


def render_json(obj):
    """Dumps the object in the `obj` param to a JSON string and returns it
    via HTTP using the application/json mimetype."""

    return HttpResponse(simplejson.dumps(obj),
                        mimetype="application/json")


def redirect(request, link='/', override_request=False):
    """A smarter redirect which takes the redirection target from
    GET['redirect_to'] or falls back to the URL from the `link` param.
    If `override_request` is True, `link` is always followed."""

    if not override_request \
       and request.method == 'GET' \
       and 'redirect_to' in request.GET:
        link = request.GET['redirect_to']
    return HttpResponseRedirect(link)


def typical_handler(request, form_class, template, initial={},
                    initial_kwargs={}, context={}):
    """A handler for a typical form workflow:

    1. Initialize a form object.
       a.  if there are POSTed data, use them
       b.  otherwise use `initial` as initial data and `initial_kwargs` as
           keyword arguments for the form object constructor.
    2. If the form validates, save it and override the template name with
       a version using the "_complete.html" suffix.
    3. If there are errors on the form, prepare an additional `error_summary`
       string on the form object (for consumption by the template).
    4. Render the result.
    """
    if request.method == 'POST':
        _kwargs = dict(initial_kwargs)
        _kwargs['data'] = request.POST
    else:
        _kwargs = dict(initial_kwargs)
        _kwargs['initial'] = initial
    form = form_class(**_kwargs)
    if form.is_valid():
        template = template + '_complete.html'
        form.save()
    else:
        template += '.html'
        if form.errors:
            # FIXME: i18n of "Try again" below.
            error = unicode(" ".join(form.errors.setdefault('__all__',
                                    [u'Spróbuj ponownie'])))
            error = DOTS_REGEX.sub('.<br>', error,
                                   len(DOTS_REGEX.findall(error))-1)
            form.error_summary = mark_safe(error)
            del error
    ctx = dict(context)
    ctx.update(locals())
    return render(request, template, ctx)


def cut(text, length=40, trailing=" (...)"):
    """Cuts text to a predefined length and appends a trailing ellipsis for
    longer sources."""
    if not text:
        return text
    if len(text) <= length:
        trailing = ""
    return text[:length] + trailing



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2010 Łukasz Langa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""BBCode-related template tags."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from django.template import Library
from django.utils.safestring import mark_safe
from postmarkup import render_bbcode

register = Library()

@register.filter
def bbcode(text):
    """Simply encode the BBCode within the text.

    Template tag available in ``common`` app's ``bbcode`` library.

    :param text: the string to render as BBCode"""
    return mark_safe(render_bbcode(text))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2011 Łukasz Langa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""langacore.kit.django.common.templatetags
   ----------------------------------------

   Commonly useful template tags and filters."""
#FIXME: Elaborate what's included.

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from functools import partial

from django.template import Node, RequestContext, Template

from langacore.kit.cache import memoize

def _render(text, context):
    if not context or 'request' not in context:
        raise ValueError("Provided context is empty or doesn't provide "
                "REQUEST.")
    return Template(text).render(RequestContext(context['request'],
        context))

class DynamicNode(Node):
    """A Node that provides a rendering method to support dynamic arguments."""

    render = staticmethod(_render)

    @memoize(skip_first=True)
    def dynamic(self, text, load=[], prefix='', suffix=''):
        """Prepares tag attribute `text` to be rendered using the templating
        engine. Returns a callable with a single `context` attribute.

        If `load` is given, it's a list of tag/filter libraries required
        for rendering of the `text`. `prefix` and `suffix` hold boilerplate
        that needs to be added around the provided text.

        If `text` is empty, the returned callable always returns an empty
        string. In this case the templating engine is not used and the
        `prefix`, `suffix` and `load` attributes are ignored.
        """
        if text:
            load = ['{{%load {}%}}'.format(l) for l in load]
            if prefix:
                load.append(prefix)
            load.append(text)
            if suffix:
                load.append(suffix)
            text = "\n".join(load)
            return partial(_render, text)
        else:
            return lambda context: ""

    def static(self, text, prefix='', suffix=''):
        """Prepares tag attribute `text` to be rendered statically. Unless
        `text` is empty, returns a string surrounded by `prefix` and `suffix`.
        Otherwise returns an empty string."""
        if text:
            return prefix + text + suffix
        else:
            return ''
